;***************************************************************************

;* File Name		:i2c.inc

;* Title		:Software I2C
;* Author		:C.Kuehnel

;* Date	 		:10/05/97

;* Version 		:1.0

;* Target MCU		:AT90S8515
;* 

;* DESCRIPTION
;* Simple software I2C driver for AVR processors. Compatible with all AVR
;* processors: AT90S1200, AT90S8515, etc..
;***************************************************************************	.

; Primary routines:
;------------------
; i2c_setup:	Call to setup I/O bits for I2C. Must call prior to using the
;               following routines.
; i2c_write:	Call to write I2C device with byte to send in i2c_buffer.
; i2c_read:	Call to read I2C device, byte returned in i2c_buffer.
;
; Registers used by software uart
; define in calling program as needed (must be >= r16)
;---------------------------------------------------------------------------
;	bitcntr 	 	; bit counter
;	i2c_del 	 	; delay loop variable
;	i2c_buffer	 	; data byte transmit/received by I2C
;	bus_check		; check for bus idle
;
;
; Start and stop condition need setup resp. hold times of 4.7 us minimum.
; Clock Lo and Hi period must be 4.7 us minimum too.
; Required delay is build by a down-counter loop initialized with 4 for 4 MHz
; clock frequency.
;

i2c_wait:	ldi	i2c_del, i2c_delay	; (1) + (3)
_i2c_wait_yet:	dec	i2c_del			; (1)
		brne	_i2c_wait_yet		; (1/2)
		ret				; (4)
;
; Set SDA and SCL to Hi level and switches SDA in a passive state for multi-master
; operation afterwards.
;
i2c_setup:	sbi	i2c_ddr, scl		; set SCL to output
		sbi	i2c_ddr, sda		; set SDA to output
		sbi	i2c_port,scl		; set SCL Hi
		sbi	i2c_port,sda		; set SDA Hi
		cbi	i2c_ddr, scl		; set SCL to input (pull-up active)
		cbi	i2c_ddr, sda		; set SDA to input (pull-up active)
		rcall	i2c_wait
		ret
;	
; Sends an I2C Start Condition to the I2C bus after check and wait for idle bus,
; means SDA = SCL = Hi.
;
i2c_start:	sbi	i2c_port,scl		; set SCL Hi
		sbi	i2c_port,sda		; set SDA Hi
		cbi	i2c_ddr, scl		; set SCL to input (pull-up active)
		cbi	i2c_ddr, sda		; set SDA to input (pull-up active)
		in	bus_check, i2c_pins		; check I2C bus
		andi	bus_check, (1<<SDA)|(1<<SCL) 	; mask SDA and SCL
		ldi	temp, (1<<SDA)|(1<<SCL)		
		cp	bus_check, temp		; test for SDA=Hi and SCL=Hi
		brne	i2c_start
		sbi	i2c_ddr, scl		; set SCL to output
		sbi	i2c_ddr, sda		; set SDA to output
		cbi	i2c_port, sda		; set SDA to Lo
		rcall	i2c_wait		
		cbi	i2c_port, scl		; set SCL to Lo
		rcall	i2c_wait
		ret

i2c_stop:	sbi	i2c_port, scl		; set SCL to Hi
		rcall	i2c_wait
		sbi	i2c_port, sda		; set SDA to Hi
		cbi	i2c_ddr, scl		; set SCL to input (pull-up active)
		cbi	i2c_ddr, sda		; set SDA to input (pull-up active)
		rcall	i2c_wait
		ret
;
; Queries for I2C Achknowledge (SDA=Lo)
; T Flag in SREG serves as an error flag
; it is set if there was NAK and cleared if there was an ACK
;
i2c_ack:	clt				; clear error flag
		sbi	i2c_port,sda		; set SDA Hi
		cbi	i2c_ddr, sda		; set SDA to input (pull-up active)
		sbi	i2c_port, scl		; set SCL to Hi
		ldi	i2c_del, i2c_delay	; (1) + (3)
_ack_wait_yet:	dec	i2c_del			; (1)
		brne	_ack_wait_yet		; (1/2)
		sbic	i2c_pins, sda		; reads SDA
		set				; set error flag when NAK
		cbi	i2c_port, scl		; set SCL to Lo
		sbi	i2c_port, sda		; set SDA to Hi
		ldi	i2c_del, i2c_delay	; (1) + (3)
_ack_wait_yet_2:dec	i2c_del			; (1)
		brne	_ack_wait_yet_2		; (1/2)
		ret
;
; Writes eight bits to I2C bus and queries the acknoledge bit from receiver
;
i2c_write:	sbi	i2c_ddr, sda		; set SDA to output
		cbi	i2c_port, sda		; set SDA to Lo
		ldi	bitcntr, 8		; eight bits to write
_wrt_bit:	sbrc	i2c_buffer, 7		; if MSB = Hi then
		sbi	i2c_port, sda		; set SDA to Hi
		nop
		sbi	i2c_port, scl		; set SCL to Hi
		rcall	i2c_wait
		cbi	i2c_port, scl		; set SCL to Lo
		cbi	i2c_port, sda		; set SDA to Lo
		rcall	i2c_wait
		lsl	i2c_buffer		; shift i2c_buffer one bit left
		dec	bitcntr
		brne	_wrt_bit
		rcall	i2c_ack			; query acknowledge
		ret
;
; Reads eight bits from I2C bus 
;
i2c_read:	ldi	bitcntr, 8		; eight bits to read
		clr	i2c_buffer		; clear I2C buffer
		clc				; clear carry flag
		cbi	i2c_ddr, sda		; set SDA to input
_rd_bit:	sbi	i2c_port, scl		; set SCL to Hi
		sbic	i2c_pins, sda		; if SDA = Hi then
		sec				;                  set carry flag
		rol	i2c_buffer		; rotate i2c_buffer one bit left
		rcall	i2c_wait
		cbi	i2c_port, scl		; set SCL to Lo
		rcall	i2c_wait
		dec	bitcntr
		brne	_rd_bit
		sbi	i2c_port, sda		; set SDA to Hi
		sbi	i2c_ddr, sda		; set SDA to output
		rcall	i2c_wait
		sbi	i2c_port, scl		; set SCL to Hi
		rcall	i2c_wait
		cbi	i2c_port, scl		; set SCL to Lo
		ret